#
#
# Purpose:
#    This Perl include

use warnings;
use DBI;

our $utils_path = $ENV{'CODENDI_UTILS_PREFIX'} || "/usr/share/codendi/src/utils";
require $utils_path."/include.pl";

##### Fix issues with Codex permissions #####
sub get_user_id_from_ldap{
    &db_connect;
    our $cgi;
    our $username    = $dbh->quote($cgi->remote_user);
    my  $user_query = "SELECT user_id FROM user WHERE user_name = $username";
    my  $user_sth   = $dbh->prepare($user_query);
    my  $user_res   = $user_sth->execute();
    if (my @user_row = $user_sth->fetchrow_array) {
        our $user_id = @user_row[0];
        return $user_id ;
    } else {
        return 0;
    }
}

sub codex_membership{
    our @projects    = ();
    &db_connect;
    my $user_id      = $dbh->quote(get_user_id_from_ldap);
    my $groups_query = "SELECT groups.unix_group_name
                          FROM groups,user_group
                          WHERE user_group.user_id= $user_id
                          AND groups.group_id=user_group.group_id
                          AND groups.status='A'
                          AND groups.type='1'";
    my $groups_sth   = $dbh->prepare($groups_query);
    $res             = $groups_sth->execute();
    if ($groups_sth->rows >= 1) {
        while (my @row = $groups_sth->fetchrow_array()) {
            my $project       = shift @row;
            my $prj_unix_name =  lc ($project);
            push @projects, $prj_unix_name;
        }
    }
    return @projects;
}

sub get_project_unix_name_from_repo {
    my $repository = $_[0];
    my @liste = split(/\//,$repository);
    my $project_unix_name = shift @liste;
    return $project_unix_name;
}

sub get_repository_id_from_repo {
    &db_connect;
    my $repo_path  = $dbh->quote($_[0]);
    my $repo_query = "select repository_id from plugin_git where repository_path = $repo_path";
    my $repo_sth   = $dbh->prepare($repo_query);
    my $repo_res   = $repo_sth->execute();
    if (my @repo_row = $repo_sth->fetchrow_array) {
        our $repo_id = @repo_row[0];
        return $repo_id ;
    } else {
        return 0;
    }
}

sub get_user_status{
    use Data::Dumper;
    &db_connect;
    my $user_id    = $dbh->quote($_[0]);
    my $user_query = "SELECT status FROM user WHERE user_id = $user_id ";
    my $user_sth   = $dbh->prepare($user_query);
    my $user_res   = $user_sth->execute();
    if (my @user_row = $user_sth->fetchrow_array){
       return @user_row[0];
    }
}

sub user_is_member{
    # Missing check on user status ... IN ("A", "R")
    # @todo: Factorize with user_is_project_admin sub
    my $dbh             = db_connect;
    my $user_id         = $dbh->quote($_[0]);
    my $group_unix_name = $dbh->quote($_[1]);
    my $member_query     = "SELECT user_id
                          FROM  groups,user_group
                          WHERE user_group.user_id= $user_id
                          AND groups.group_id = user_group.group_id
                          AND groups.unix_group_name = $group_unix_name";
    my $member_sth       = $dbh->prepare($member_query);
    my $member_res       = $member_sth->execute();
    if ($member_sth->rows >= 1) {
        return 1;
    } else {
        return 0;
    }
    return 1;
}

sub user_is_project_admin{
    # Missing check on user status ... IN ("A", "R")
    &db_connect;
    my $user_id         = $dbh->quote($_[0]);
    my $group_unix_name = $dbh->quote($_[1]);
    my $admin_query     = "SELECT user_id
                           FROM  groups,user_group
                           WHERE user_group.user_id= $user_id
                           AND groups.group_id = user_group.group_id
                           AND groups.unix_group_name = $group_unix_name
                           AND user_group.admin_flags = 'A'";
    my $admin_sth       = $dbh->prepare($admin_query);
    my $admin_res       = $admin_sth->execute();
    if ($admin_sth->rows >= 1) {
        return 1;
    } else {
        return 0;
    }
}

sub get_repo_permissions_from_DB {
    # Missing check for Restricted Users
    &db_connect;
    my $user_id       = get_user_id_from_ldap;
    my $repository        = $_[0];
    my $project_unix_name = $_[1];

    # retrieve repository id from its name / path
    my $objectId   = $dbh->quote(get_repository_id_from_repo($repository));

    # Retrieve ugroup list having read on given git repositories

    my $where_clause = " object_id = $objectId AND permission_type = 'PLUGIN_GIT_READ'";
    my $perms_sql    = 'SELECT ugroup_id FROM permissions WHERE '.$where_clause;
    my $perms_sth    = $dbh->prepare($perms_sql);
    my $perms_res    = $perms_sth->execute();

    if($perms_sth->rows > 0){

        while (my @row = $perms_sth->fetchrow_array()){
            # foreach Ugroup having permissions on given gitolite repo
            # check if cgi user is member
            my $ugroup_id = @row[0];

            #check dynamic ugroups first
            if($ugroup_id == 100){
                #Read granted to ugroup_nobody_name_key
                return 0;
            }elsif($ugroup_id == 1){
                # Read granted to ugroup_anonymous_users_name_key
                return 1;
            }elsif($ugroup_id == 2){
                # Read granted to ugroup_registered_users_name_key
                # Read is granted since users authenticated thru ST-LDAP <=> Registered user
                return 1;
            }elsif($ugroup_id == 3){
                # Read granted to ugroup_project_members_name_key
                return user_is_member($user_id, $project_unix_name);
            }elsif($ugroup_id == 4){
                #Read granted to ugroup_project_admins_name_key;
                return user_is_project_admin($user_id, $project_unix_name);
            }else{
                my $ugroup_id = $dbh->quote($ugroup_id);
                my $user_id   = $dbh->quote($user_id);
                # Default ugroup membership
                my $ugroup_sql = "SELECT user_id from ugroup_user where ugroup_id= $ugroup_id
                     and user_id = $user_id";
                my $ugroup_sth = $dbh->prepare($ugroup_sql);
                my $ugroup_res = $ugroup_sth->execute();
                if($ugroup_sth->rows > 0){
                    return 1;
                } else {
                    return 0;
                }
            }
        }
    } else {
        # No gitolite permission in Codex DB => Default gitolite permissions are applied
        # @todo: retrieve perms from repo conf
        return 1;
    }
}

sub user_can_see {
    my $repository            = $_[0];
    my $project_unix_name     = get_project_unix_name_from_repo($repository);
    my $repository_permission = get_repo_permissions_from_DB($repository, $project_unix_name);

    my @group_array = codex_membership;
    if(grep(/^$project_unix_name$/, @group_array) && ($repository_permission != 0)){
        return 1;
    }else{
        return 0;
    }
 }